package ch.polgrabia.demos.server;

import io.vertx.core.Handler;
import io.vertx.core.buffer.Buffer;
import io.vertx.core.http.ServerWebSocket;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class WebsocketServerHandler implements Handler<ServerWebSocket> {
    private static final Logger logger = LoggerFactory.getLogger(WebsocketServerHandler.class);

    private static final ConcurrentHashMap<String, Set<ServerWebSocket>>
            channelSocketMapping = new ConcurrentHashMap<>();

    @Override
    public void handle(ServerWebSocket serverWebSocket) {
        serverWebSocket.writeBinaryMessage(Buffer.buffer("Hello"));
        serverWebSocket.handler(new WebsocketMessageHandler(serverWebSocket));
        String channel = serverWebSocket.path();
        Set<ServerWebSocket> sockets = channelSocketMapping.computeIfAbsent(channel, (k) -> {
            Set<ServerWebSocket> s = new HashSet<>();
            s.add(serverWebSocket);
            return s;
        });


        serverWebSocket.closeHandler(aVoid -> {
            int id = System.identityHashCode(serverWebSocket);
            logger.info("[{}] Closing web socket with id {}", channel, id);
            Set<ServerWebSocket> serverWebSockets = channelSocketMapping.get(channel);
            if (serverWebSockets != null) {
                boolean removed = serverWebSockets
                        .remove(serverWebSocket);
                if (!removed) {
                    logger.warn("Server websocket channel {} was not found in the channel set of sockets." +
                            " It should never happen", channel);
                }
            } else {
                logger.warn("Channel {} not found. It should never happen", channel);
            }
        });

        sockets.add(serverWebSocket);
    }

    static class WebsocketMessageHandler implements Handler<Buffer> {

        private final ServerWebSocket serverWebSocket;

        public WebsocketMessageHandler(ServerWebSocket serverWebSocket) {
            this.serverWebSocket = serverWebSocket;
            this.serverWebSocket.setWriteQueueMaxSize(0);
        }


        @Override
        public void handle(Buffer s) {
            String channel = serverWebSocket.path();
            logger.info("[{}] Got message {}", channel, s);
            serverWebSocket.writeBinaryMessage(Buffer.buffer("Pong: " + s.toString() + "\n")); // TBD send to kafka
        }
    }

}
